<?php
namespace Pinpoint\Feed\Model;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product;
use Pinpoint\Feed\Helper\Data;

/**
 * Class ProductFeed
 * @package Pinpoint\Feed\Model
 */
class ProductFeed extends AbstractFeed
{
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $_productFactory;

    /**
     * @var bool
     */
    protected $categoryPaths = false;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Category\Collection
     */
    protected $categoryCollection;


    /**
     * ProductFeed constructor.
     * @param \Magento\Framework\Escaper $escaper
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $collectionFactory
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $modelDate
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Framework\Stdlib\DateTime $dateTime
     * @param ProductRepositoryInterface $productRepository
     * @param Data $helper
     * @param \Magento\Framework\Filesystem\Io\File $io
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\ResourceModel\Category\Collection $categoryCollection
     * @param \Magento\Config\Model\Config\Reader\Source\Deployed\DocumentRoot|null $documentRoot
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $collectionFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $modelDate,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        ProductRepositoryInterface $productRepository,
        Data $helper,
        \Magento\Framework\Filesystem\Io\File $io,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ResourceModel\Category\Collection $categoryCollection,
        \Magento\Config\Model\Config\Reader\Source\Deployed\DocumentRoot $documentRoot = null
    ) {
        $this->_productFactory = $collectionFactory;
        $this->productRepository = $productRepository;
        $this->categoryCollection = $categoryCollection;
        parent::__construct($escaper, $filesystem, $helper, $modelDate, $request, $dateTime, $io, $storeManager, $documentRoot);
    }

    /**
     * @param $productId
     * @return mixed|string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function _getRow($productId)
    {
        $product = $this->productRepository->getById($productId, false, $this->getStoreId());

        /** @var Product $product */
        $url = $product->getProductUrl();
        $row = '<id>' . $product->getId() . '</id>';

        if ($product->getParentId()) {
            $row .= '<itemGroupId>' . $product->getParentId() . '</itemGroupId>';
        }

        $row .= '<name><![CDATA[' . $product->getData($this->helper->getProductFeedField('name', $this->getStoreId())) . ']]></name>';

        if ($image = $product->getImage()) {
                $row .= '<image>' . htmlspecialchars($this->_getMediaUrl($image));
                $row .= '</image>';
        }

        $row .= '<url>' . htmlspecialchars($url) . '</url>';
        $price = $product->getPrice()?:0;
        $row .= '<price>' . $price . '</price>';
        $finalPrice = $product->getFinalPrice()?:0;
        $row .= '<salePrice>' . $finalPrice . '</salePrice>';
        $row .= '<currency>' . $this->getStore()->getCurrentCurrency()->getCode() . '</currency>';
        $discount = $finalPrice - $price;
        $row .= '<discount>' . $discount . '</discount>';

        $row .= '<shortDesc><![CDATA[' . $product->getData($this->helper->getProductFeedField('short_description', $this->getStoreId())) . ']]></shortDesc>';
        $row .= '<desc><![CDATA[' . $product->getData($this->helper->getProductFeedField('description', $this->getStoreId())) . ']]></desc>';

        $category = $this->getLongestPathCategoryName($product->getCategoryIds());

        if ($product->getCategoryIds()) {
            $listCategory = implode(';', $this->getCategoryNames($product->getCategoryIds()));
        } else {
            $listCategory = 'KLEVU_PRODUCT';
        }

        $row .= '<category><![CDATA[' . $category . ']]></category>';
        $row .= '<listCategory><![CDATA[' . $listCategory . ']]></listCategory>';

        $inStock = $product->isAvailable()?'yes':'no';
        $row .= '<inStock>' . $inStock . '</inStock>';

        if ($product->getRating()) {
            $row .= '<rating>' . $this->convertToRatingStar($product->getRating()) . '</rating>';
        }

        $tags = $product->getData($this->helper->getProductFeedField('tags', $this->getStoreId()));
        if ($tags) {
            $row .= '<tags>' . $tags . '</tags>';
        }
        $row .= '<sku><![CDATA[' . $product->getSku() . ']]></sku>';

        return '<product>' . $row . '</product>';
    }

    /**
     * @return array|bool
     */
    protected function getCategoryPaths()
    {
        if (!$category_paths = $this->categoryPaths) {
            $category_paths = [];
            $rootId = $this->store->getRootCategoryId();
            $collection = $this->categoryCollection
                ->setStoreId($this->getStoreId())
                ->addFieldToFilter('level', ['gt' => 1])
                ->addFieldToFilter('path', ['like'=> "1/$rootId/%"])
                ->addIsActiveFilter()
                ->addNameToResult();
            foreach ($collection as $category) {
                $category_paths[$category->getId()] = [];
                $path_ids = $category->getPathIds();
                foreach ($path_ids as $id) {
                    if ($item = $collection->getItemById($id)) {
                        $category_paths[$category->getId()][] = $item->getName();
                    }
                }
            }
            $this->categoryPaths = $category_paths;
        }
        return $category_paths;
    }

    /**
     * @param $url
     * @return string
     */
    protected function _getMediaUrl($url)
    {
        return $this->_getUrl('catalog/product' . $url, \Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
    }

    /**
     * Return a list of the names of all the categories in the
     * paths of the given categories (including the given categories)
     * up to, but not including the store root.
     *
     * @param array $categories
     *
     * @return array
     */
    protected function getCategoryNames(array $categories)
    {
        $category_paths = $this->getCategoryPaths();
        $result = ["KLEVU_PRODUCT"];
        foreach ($categories as $category) {
            if (isset($category_paths[$category])) {
                if(count($category_paths[$category]) > 0) {
                    $cat_path[$category][] = implode(";",$category_paths[$category]);
                } else {
                    $cat_path[$category] = $category_paths[$category];
                }
                $result = array_merge($result, $cat_path[$category]);
            }
        }
        return array_unique($result);
    }
    /**
     * Given a list of category IDs, return the name of the category
     * in that list that has the longest path.
     *
     * @param array $categories
     *
     * @return string
     */
    protected function getLongestPathCategoryName(array $categories)
    {
        $category_paths = $this->getCategoryPaths();
        $name = "";
        foreach ($categories as $id) {
            if (isset($category_paths[$id])) {
                $name .= end($category_paths[$id]).";";
            }
        }
        return substr($name, 0, strrpos($name, ";")+1-1);
    }

    /**
     * @param $percentage
     * @return float|null
     */
    public function convertToRatingStar($percentage)
    {
        if (!empty($percentage) && $percentage!=0) {
            $start = $percentage * 5;
            return round($start/100, 2);
        } else {
            return null;
        }
    }

    /**
     * @return mixed
     */
    public function getFilename()
    {
        return $this->helper->getProductFeedFilename($this->getStoreId());
    }

    /**
     * @return mixed
     */
    public function getFilePath()
    {
        return $this->helper->getProductFeedFilepath($this->getStoreId());
    }

    /**
     * @return $this|mixed
     * @throws \Magento\Framework\Exception\FileSystemException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function generateXml()
    {
        if (!$this->helper->isProductFeedEnabled($this->getStoreId())) {
            return $this;
        }

        $this->prepareFile();
        $this->_lineCount = 0;

        $this->_initSitemapItems();
        $storeId = $this->getStoreId();

        /** @var $sitemapItem \Magento\Framework\DataObject */
        $collection = $this->_productFactory->create()->addStoreFilter($storeId);

        foreach ($collection->getAllIds() as $itemId) {
            $xml = $this->_getRow(
                $itemId
            );

            if (!$this->_fileSize) {
                $this->_createFeed();
            }

            $this->_writeRow($xml);

            $this->_lineCount++;
            $this->_fileSize += strlen($xml);
        }


        $this->_finalizeFile();

        return $this;
    }
}
